//
// C++ Interface: caltrack_reader_common
//
// Description: 
//
//
// Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef CALTRACK_READER_COMMON_H
#define CALTRACK_READER_COMMON_H

#include "float.h"
#include "math.h"

#include "c_hdf_reader.h"

/**
 * @file caltrack_reader_common.h
 * Functions used to read caltrack files common to all products
 */

/**
 * @def NB_INPUT_FILE_MAX maximum number of input files that have been used to build a caltrack file
 */
#define NB_INPUT_FILE_MAX 20
#define NB_AEROSOL_LAYER_CALIOP 8
#define NB_PARASOL_DIRECTION 16
#define NB_CLOUDSAT_VERTICAL_BINS 125

/**
 * @brief default scaling function for HDF data : scaled_value = (bin_value - scale_offset) * scale_factor
 * @param invalue value to scale
 * @param scale_factor slope
 * @param scale_offset offset
 * @return the scaled data value
 */
double default_scale_function ( const double invalue, const double scale_factor, const double scale_offset );

/**
 * @brief scaling function for PARASOL data : scaled_value = bin_value * scale_factor + scale_offset
 * @param invalue value to scale
 * @param scale_factor slope
 * @param scale_offset offset
 * @return the scaled data value
 */
double parasol_scale_function ( const double invalue, const double scale_factor, const double scale_offset );

/**
 * @brief read the data of one dataset and apply the scaling to it
 * @warning if the caller delegate the allocation of the data buffer (@a sds_data) to the funtion by setting it to NULL, the data will be accessed lineary. So, if the dataset has 2 dimensions, to access the value at (i,j) you will do sds_data[j*dim_x+i] instead of the usual sds_data[j][i] (with dim_x the size of the x dimension). If you want to use an indexation as [j][i], the caller must manage the allocation
 * @param sd_id [IN] an SD id as returned by the open_hdf_file function
 * @param sds_name [IN] name of the sds to read
 * @param sds_data [OUT] the buffer where is stored the read data. If set to NULL, the method will manage the memory allocation
 * @param start [IN] Array specifying the starting location from where data is read. Set it to NULL to read all the dataset
 * @param stride [IN] Array specifying the interval between the values that will be read along each dimension. Set it to NULL to read all the contigous data
 * @param edges [IN] Array specifying the number of values to be read along each dimension. Set it to NULL to read all the dataset
 * @param scale_function [IN] pointer to the scaling function
 * @return a pointer to the eventually allocate data buffer if successfull, NULL in case of error
 */
double * read_custom_scale_sds_data_by_id( const int32 sd_id, const char* sds_name, double * sds_data,
                                        int32 *start, int32 *stride, int32 *edges,
                                        double (*scale_function) (const double, const double, const double) );

/**
 * @brief apply a non HDF standard scaling function to an array of calibrated data
 * @param indata [IN] array of calibrated binary data
 * @param indata_size [IN] number of elements in indata
 * @param indata_typecode [IN] indata HDF typecode
 * @param scale_factor [IN] scaling factor
 * @param scale_offset [IN] scaling offset
 * @param scale_function [IN] pointer to the scaling function
 * @param outdata [OUT] output scaled data
 * @return a pointer to the eventually allocate data buffer if successfull, NULL in case of error
 */
double * custom_scale_data (  const void* indata, const int32 indata_size, const int32 indata_typecode,
                    const double scale_factor, const double scale_offset,
                    double (*scale_function) (const double, const double, const double),
                    double * outdata );

/**
 * @brief read the list of input files of a product that have been used to build a caltrack product
 * @param filename [IN] the full path to the file to read
 * @param product [IN] name of the product
 * @param v_files[][] array of files names
 * @param nb_file number of files of v_files
 * @return 0 if successfull, -1 in case of error
 */
int32 get_v_input_files( const char* filename, const char* product, char v_files[][MAX_NC_NAME], int32 *nb_file );

/**
 * @brief read the list of input files of a product that have been used to build a caltrack product
 * @param sd_id [IN] an SD id as returned by the open_hdf_file function
 * @param product [IN] name of the product
 * @param v_files[][] array of files names
 * @param nb_file number of files of v_files
 * @return 0 if successfull, -1 in case of error
 */
int32 get_v_input_files_by_id( const int32 sd_id, const char* product, char v_files[][MAX_NC_NAME], int32 *nb_file );

/**
 * @brief retrieve an input file name of a given product, using its index
 * @param sd_id [IN] an SD id as returned by the open_hdf_file function
 * @param product [IN] name of the product
 * @param i_file [IN] index of the file, as set in caltrack's "Input_File_Index" datasets
 * @param infile[] [OUT] name of the input file used to produce the caltrack file
 * @return 0 if successfull, -1 in case of error
 */
int32 get_input_file_by_id ( const int32 sd_id, const char* product, const int32 i_file, char infile[MAX_NC_NAME] );

/**
 * @brief check if the given variable is a Caltrack fill value
 * @param p_value [IN] pointer to the value to test
 * @param typecode [IN] an HDF typecode
 * @return 1 if data is a fill value, 0 if not, -1 in case of problem
 */
int32 is_fill_value ( const void* p_value, const int32 typecode );

#endif
