/*
*
* Description: library for reading caltrack PARASOL L1 data records
*
* History :
*   v0.0.0 2008/02/25 : creation
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) Centre de Gestion et de Traitement de Donnes (CGTD) ICARE 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#ifndef CALTRACK_READER_PARASOL_L1_H
#define CALTRACK_READER_PARASOL_L1_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_PARASOL_L1 stores the variables issued of the PARASOL L1 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    /** Index [i_row,icol] in the PARASOL grid of the coincident Pixel in the input product */
    int16  input_index                  [2];
    /** Number of available viewing directions Ndir in the following 1<=id<=Ndir */
    uint8  nb_available_viewing_direction;
    /** Solar Azimuth Angle */
    float64  solar_azimuth_angle;
    /** Pixel Data Quality Index for each direction. Also known as DQX */
    uint16 pixel_quality_index          [NB_PARASOL_DIRECTION];
    /** Solar Zenith Angle for each direction */
    float64 solar_zenith_angle           [NB_PARASOL_DIRECTION];
    /** View Zenith Angle for filter #8 (670P2), for each direction */
    float64 view_zenith_angle_670p2      [NB_PARASOL_DIRECTION];
    /** Relative Azimuth Angle for filter #8 (670P2), for each direction */
    float64 relative_azimuth_angle_670p2 [NB_PARASOL_DIRECTION];
    /** Relative variation of viewing geometry between the filters, for each direction. See the appendix C of the \"PARASOL Level-1 Product Data Format and User Manual\" for details */
    float64   delta_thetav_cosphi          [NB_PARASOL_DIRECTION];
    /** Relative variation of viewing geometry between the filters, for each direction. See the appendix C of the \"PARASOL Level-1 Product Data Format and User Manual\" for details */
    float64   delta_thetav_sinphi          [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 443NP, for each direction */
    float64  normalized_radiance_443np    [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 490P, for each direction */
    float64  normalized_radiance_490p     [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 1020NP, for each direction */
    float64  normalized_radiance_1020np   [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 565NP, for each direction */
    float64  normalized_radiance_565np    [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 670P, for each direction */
    float64  normalized_radiance_670p     [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 763NP, for each direction */
    float64  normalized_radiance_763np    [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 765NP, for each direction */
    float64  normalized_radiance_765np    [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 865P, for each direction */
    float64  normalized_radiance_865p     [NB_PARASOL_DIRECTION];
    /** Normalized Radiance for Channel 910NP, for each direction */
    float64  normalized_radiance_910np    [NB_PARASOL_DIRECTION];
    /** Second component of Stokes vector (Q) for Channel 490P, for each direction */
    float64  q_stokes_490p                [NB_PARASOL_DIRECTION];
    /** Second component of Stokes vector (Q) for Channel 670P, for each direction */
    float64  q_stokes_670p                [NB_PARASOL_DIRECTION];
    /** Second component of Stokes vector (Q) for Channel 865P, for each direction */
    float64  q_stokes_865p                [NB_PARASOL_DIRECTION];
    /** Third component of Stokes vector (U) for Channel 490P, for each direction */
    float64  u_stokes_490p                [NB_PARASOL_DIRECTION];
    /** Third component of Stokes vector (U) for Channel 670P, for each direction */
    float64  u_stokes_670p                [NB_PARASOL_DIRECTION];
    /** Third component of Stokes vector (U) for Channel 865P, for each direction */
    float64  u_stokes_865p                [NB_PARASOL_DIRECTION];
} caltrack_record_PARASOL_L1;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_PARASOL_L1 ( const char* filename, const int i_rec, caltrack_record_PARASOL_L1 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_PARASOL_L1 ( const caltrack_record_PARASOL_L1 const *record );

#endif
