/*
*  C Implementation: caltrack_reader_common
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader_common.h"

double default_scale_function ( const double invalue, const double scale_factor, const double scale_offset ) {
    return scale_factor * ( invalue - scale_offset ) ;
};

double parasol_scale_function ( const double invalue, const double scale_factor, const double scale_offset ) {
    return scale_factor * invalue + scale_offset;
};

double * read_custom_scale_sds_data_by_id( const int32 sd_id, const char * sds_name, double * sds_data,
                                        int32 * start, int32 * stride, int32 * edge,
                                        double (* scale_function)( const double , const double , const double ) ) {
    int   i         = 0;
    intn  status    = FAIL; // return code of hdf functions
    int32 nb_data   = 0;    // number of data to read
    int8  is_start_set = 1; // tell if the start parameter has been set by the caller
    int8  is_edge_set = 1; // tell if the edge parameter has been set by the caller
    float64 cal = 0.; // calibration factor
    float64 cal_err = 0.; // calibration error
    float64 offset = 0.; // calibration offset
    float64 offset_err = 0.; // calibration offset error
    int32   data_type = -1; // Data Type code
    int   is_inner_allocation = 0; // tells if the method does the allocation of sds_data or not
    int   is_error_occur = 0; // tells if an error occurs after the allocation of sds_data

    SDS_INFO sds_info;     // infos of the SDS to read
    void* unscaled_sds_data = NULL; // buffer that will store unscaled data

    /* read infos of the sds : rank, data_type and dimensions size */
    sds_info.v_attr = NULL;
    status = get_sds_info_by_id (sd_id,sds_name,&sds_info);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Failed to read the data\n",__FILE__,__LINE__);
        return NULL;
    }

    /* set if the function have to guess the number of data to read */
    if ( start == NULL ) is_start_set = 0;
    if ( edge  == NULL ) is_edge_set  = 0;

    /* eventually guess the number of data to read */
    status = set_start_stride_edge(sd_id,sds_name,&start,stride,&edge);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Failed to guess number of data to read in SDS %s\n",__FILE__,__LINE__,sds_name);
        sds_data = NULL;
        goto exit;
    }

    /* compute the number of data to read and so on the number of data of the buffer */
    nb_data = 1;
    for ( i = 0 ; i < sds_info.rank ; ++i )
        nb_data*=edge[i];

    /* read unscaled data */
    unscaled_sds_data = read_sds_binary_data_by_id(sd_id,sds_name,unscaled_sds_data,start,stride,edge);
    if (unscaled_sds_data == NULL) {
        fprintf(stderr,"%s[L%d] Failed to read data of SDS %s\n",__FILE__,__LINE__,sds_name);
        sds_data = NULL;
        goto exit;
    }

    /* read the scaling of the SDS */
    status = get_sds_calibration_by_id(sd_id, sds_name, &cal, &cal_err, &offset, &offset_err, &data_type);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Failed to read the calibration of SDS %s\n",__FILE__,__LINE__,sds_name);
        sds_data = NULL;
        goto exit;
    }

    /* eventually allocate scaled data buffer */
    if ( sds_data == NULL) {
        is_inner_allocation = 1;
        sds_data = (double*) malloc ( sizeof(double) * nb_data );
        if ( sds_data == NULL) {
            fprintf(stderr,"%s[L%d] Memory allocation error\n",__FILE__,__LINE__);
            is_error_occur = 1;
            goto exit;
        }
    }

    /* apply scaling */
    sds_data = custom_scale_data (  unscaled_sds_data, nb_data, data_type, cal, offset, scale_function, sds_data );
    if ( sds_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to scale value\n",__FILE__,__LINE__);
        is_error_occur = 1;
        goto exit;
    }

exit:
    /* free eventual allocations */
    free_sds_info(&sds_info);

    if ( is_start_set == 0 ) {
        free(start);
        start = NULL;
    }
    if ( is_edge_set == 0 ) {
        free(edge);
        edge = NULL;
    }

    /* free unscaled data buffer */
    free_typed_data(unscaled_sds_data,sds_info.data_type);

    /* in case of error, free inner allocated data buffer */
    if ( is_error_occur && is_inner_allocation ) {
        free(sds_data);
        sds_data=NULL;
    }

    return sds_data;
}

double * custom_scale_data(    const void* indata, const int32 indata_size, const int32 indata_typecode,
                            const double scale_factor, const double scale_offset,
                            double (*scale_function) (const double, const double, const double),
                            double * outdata ) {
    double bin_val;
    /* generic iterator */
    int i = 0;
    int   is_inner_allocation = 0; // tells if the method does the allocation of sds_data or not
    int   is_error_occur = 0; // tells if an error occurs after the allocation of sds_data

    /* eventually allocate outdata buffer */
    if ( outdata == NULL ) {
        is_inner_allocation = 1;
        (outdata) = (double*) malloc ( sizeof(double) * indata_size );
        if ( outdata == NULL ) {
            fprintf(stderr,"%s[L%d] Memory Allocation Error\n",__FILE__,__LINE__);
            is_error_occur = 1;
            goto exit;
        }
    }

    /* apply scaling to all data */
    for ( i=0 ; i<indata_size ; ++i ) {
        if ( is_fill_value( indata + i * get_data_type_size(indata_typecode), indata_typecode ) ) {
            // data is a fill value
            outdata[i] = -DBL_MAX;
        } else {
            // binary value to double
            switch (indata_typecode) {
                case ( DFNT_UCHAR8 ) :
                    bin_val = (double)( * ( (uchar8*)(indata)+i ) ); break;
                case ( DFNT_UINT8  ) :
                    bin_val = (double)( * ( (uint8*)(indata)+i ) ); break;
                case ( DFNT_CHAR8  )  :
                    bin_val = (double)( * ( (char8*)(indata)+i ) ); break;
                case ( DFNT_INT8   )  :
                    bin_val = (double)( * ( (int8*)(indata)+i ) ); break;
                case ( DFNT_UINT16   ):
                    bin_val = (double)( * ( (uint16*)(indata)+i ) ); break;
                case ( DFNT_INT16   ):
                    bin_val = (double)( * ( (int16*)(indata)+i ) ); break;
                case ( DFNT_UINT32   ):
                    bin_val = (double)( * ( (uint32*)(indata)+i ) ); break;
                case ( DFNT_INT32   ):
                    bin_val = (double)( * ( (int32*)(indata)+i ) ); break;
                case ( DFNT_FLOAT32   ):
                    bin_val = (double)( * ( (float32*)(indata)+i ) ); break;
                case ( DFNT_FLOAT64   ):
                    bin_val = (double)( * ( (float64*)(indata)+i ) ); break;
                default              :
                    fprintf(stderr,"%s[L%d] Invalid HDF typecode %d\n",__FILE__,__LINE__,(int)(indata_typecode));
                    is_error_occur = 1;
                    goto exit; break;
            }
            // apply scaling
            outdata[i] = (*scale_function)( bin_val, scale_factor, scale_offset );
        }
    }
exit:

    /* in case of error, free inner allocated data buffer */
    if ( is_error_occur && is_inner_allocation ) {
        free(outdata);
        outdata=NULL;
    }

    return outdata;
}

int32 is_fill_value( const void * p_value, const int32 typecode ) {
    switch (typecode) {
        case ( DFNT_UCHAR8 ) :
        case ( DFNT_UINT8  ) :
            return *(uint8 *)  (p_value) == UCHAR_MAX ; break;
        case ( DFNT_CHAR8  )  :
        case ( DFNT_INT8   )  :
            return *(int8 *)   (p_value) == SCHAR_MIN ; break;
        case ( DFNT_UINT16   ):
            return *(uint16 *) (p_value) == USHRT_MAX ; break;
        case ( DFNT_INT16   ):
            return *(int16 *)  (p_value) == SHRT_MIN  ; break;
        case ( DFNT_UINT32   ):
            return *(uint32 *) (p_value) == ULONG_MAX ; break;
        case ( DFNT_INT32   ):
            return *(int32 *)  (p_value) == LONG_MIN  ; break;
        case ( DFNT_FLOAT32   ):
            return  fabs ( *(float32 *)(p_value)-DBL_MIN ) < FLT_EPSILON  ; break;
        case ( DFNT_FLOAT64   ):
            return  fabs ( *(float64 *)(p_value)-LDBL_MIN ) < DBL_EPSILON  ; break;
        default              :
            fprintf(stderr,"%s[L%d] Invalid HDF typecode %d\n",__FILE__,__LINE__,(int)(typecode));
            return FAIL; break;
    }
}

int32 get_input_file_by_id( const int32 sd_id, const char * product, const int32 i_file, char infile[MAX_NC_NAME] ) {
    /* HDF functions return code */
    int32 status = FAIL;

    /* array of input files */
    char v_files[NB_INPUT_FILE_MAX][MAX_NC_NAME];
    /* number of input files */
    int32 nb_file = -1;

    if (i_file != SHRT_MIN) {
        // if the file index is valid : set the input file corresponding to it
        status = get_v_input_files_by_id( sd_id, product , v_files, &nb_file);
        if (status == FAIL) { // read error
            fprintf(stderr,"%s[L%d] Fail to read the input files of product %s in %s\n",__FILE__,__LINE__,product,infile);
            return FAIL;
        }
        strcpy(infile,v_files[i_file]);
    } else {
        // invalid file -> reset
        memset(infile,0,MAX_NC_NAME);
    }
    return SUCCEED;
}

int32 get_v_input_files( const char * filename, const char * product,
                         char v_files[][MAX_NC_NAME], int32 * nb_file ) {
    int32 err     = SUCCEED; // returned error code
    int32 sd_id   = FAIL;    // SD interface identifier
    int32 status  = FAIL;    // return code of hdf functions

    /* open HDF file for reading */
    sd_id = open_hdf_file ( filename );
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Failed to open the file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    /* read input files */
    status = get_v_input_files_by_id(sd_id, product, v_files, nb_file);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Can not retrieve the input files for product %s in caltrack file %s\n",__FILE__,__LINE__,product,filename);
        err = FAIL;
        goto exit;
    }

exit:
    /* close the file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] close hdf file failed\n",__FILE__,__LINE__);
        err = FAIL;
    }
    return err;


}

int32 get_v_input_files_by_id( const int32 sd_id, const char * product, char v_files[][MAX_NC_NAME], int32 * nb_file ) {
    int32 err     = SUCCEED; // returned error code
    int32 status  = FAIL;    // return code of hdf functions
    ATTR  attr; // file attribute reading structure
    char * p_char = NULL; // generic character pointer
    char * s_input_files = NULL; // 

    /* init attr data buffer */
    attr.value = NULL;

    /* read all input files */
    status = get_file_attr_by_id(sd_id, "Input Files", &attr);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Can not retrieve the input files\n",__FILE__,__LINE__);
        err = FAIL;
        goto exit;
    }

    /* parse the read string for extracting the only the files for desired product */
    s_input_files = (char*)(attr.value);
    p_char = strstr (s_input_files,product);
    if ( p_char == NULL ) {
        // product not found
        fprintf(stderr,"%s[L%d] Can not retrieve input files of product %s\n",__FILE__,__LINE__,product);
        err = FAIL;
        goto exit;
    }
    // extract files by files
    int i = 0;
    p_char = strtok (p_char,"\n");
    p_char = strtok (NULL,"\n"); // go end of line
    while ( p_char != NULL && p_char[0] != '*' && p_char[0] != '\n') {
        // check iterator validity
        if ( i>= NB_INPUT_FILE_MAX ) {
            fprintf(stderr,"%s[L%d] Exceed buffer size\n",__FILE__,__LINE__);
            err = FAIL;
            goto exit;
        }
        // copy string
        strcpy(v_files[i],p_char+5);
        p_char = strtok (NULL,"\n");
        ++i;
    }
    *nb_file = i;

exit:
    /* free allocations */
    free_attr(&attr);

    return err;
}
