#include "caltrack_reader_modis_myd021km.h"

int32 read_band_data (const int32 sd_id, const char* sds_name, const int32 i_rec, float64 * radiance, float64 * reflectance) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* HDF binary data */
    uint16 uint16_binary_data[1];
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* radiance scaling factors */
    float64 radiance_scaling = 1.;
    float64 radiance_offset  = 1.;
    /* reflectance scaling factors */
    float64 reflectance_scaling = 1.;
    float64 reflectance_offset  = 1.;
    /* generic pointer */
    void* p_data = NULL;

    // read binary file data
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, uint16_binary_data, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        return FAIL;
    }
    // read radiance and reflectance scaling
    if (reflectance != NULL) {
        // band having reflectance
        status = get_myd021km_band_scaling_factor(sd_id, sds_name,&radiance_scaling,&radiance_offset,&reflectance_scaling,&reflectance_offset);
    } else {
        // emissive bands -> no reflectance scaling
        status = get_myd021km_band_scaling_factor(sd_id, sds_name,&radiance_scaling,&radiance_offset,NULL,NULL);
    }
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read scaling factors of SDS %s\n",__FILE__,__LINE__,sds_name);
        return FAIL;
    }

    // apply scaling
    if (uint16_binary_data[0] == USHRT_MAX) {
        // fill value
        (*radiance) = -DBL_MAX;
        if ( reflectance != NULL ) {
            (*reflectance) = -DBL_MAX;
        }
    } else {
        (*radiance) =  ( (float64)(uint16_binary_data[0]) - radiance_offset ) * radiance_scaling ;
        if ( reflectance != NULL ) {
            (*reflectance) = ( ( (float64)(uint16_binary_data[0]) - reflectance_offset ) * reflectance_scaling );
        }
    }
    return SUCCEED;
}

int32 get_myd021km_band_scaling_factor( const int32 sd_id, const char * sds_name, 
                                        float64 * radiance_scale, float64 * radiance_offset,
                                        float64 * reflectance_scale, float64 * reflectance_offset ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* SDS attributes structure */
    ATTR sds_attr;
    sds_attr.value = NULL;

    // read radiance and reflectance scaling
    status = get_sds_attr_by_id(sd_id, sds_name, "radiance_scale",&sds_attr);
    *radiance_scale = *(float64*)(sds_attr.value);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read attribut %s of SDS %s\n",__FILE__,__LINE__,"radiance_scale",sds_name);
        err = FAIL;
        goto exit;
    }

    status = get_sds_attr_by_id(sd_id, sds_name, "radiance_offset",&sds_attr);
    *radiance_offset = *(float64*)(sds_attr.value);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read attribut %s of SDS %s\n",__FILE__,__LINE__,"radiance_offset",sds_name);
        err = FAIL;
        goto exit;
    }
    
    if ( reflectance_scale != NULL && reflectance_offset != NULL ) {
        // only for non Emissive bands
        status = get_sds_attr_by_id(sd_id, sds_name, "reflectance_scale",&sds_attr);
        *reflectance_scale = *(float64*)(sds_attr.value);
        if ( status == FAIL ) {
            fprintf(stderr,"%s[L%d] Fail to read attribut %s of SDS %s\n",__FILE__,__LINE__,"reflectance_scale",sds_name);
            err = FAIL;
            goto exit;
        }

        status = get_sds_attr_by_id(sd_id, sds_name, "reflectance_offset",&sds_attr);
        *reflectance_offset = *(float64*)(sds_attr.value);
        if ( status == FAIL ) {
            fprintf(stderr,"%s[L%d] Fail to read attribut %s of SDS %s\n",__FILE__,__LINE__,"reflectance_offset",sds_name);
            err = FAIL;
            goto exit;
        }
    }

exit:
    /* free allocations */
    free_attr(&sds_attr);
    return err;
}

int read_caltrack_record_MODIS_L1( const char* filename, const int i_rec, caltrack_record_MODIS_L1 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* name of the product */
    char * product_name = "MYD021KM";
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,-1};
    /* generic pointer to data buffer */
    void* p_data = NULL;

//     int16* int16_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_Input_File_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_file_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    status = get_input_file_by_id (sd_id,product_name,record->input_file_index,record->input_file);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Fail to read the input files of product %s in %s\n",__FILE__,__LINE__,product_name,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_Input_Pixel_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->input_pixel_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_250_Aggr1km_RefSB_Band1");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_250_aggr1km_refsb_band1_radiance,&record->ev_250_aggr1km_refsb_band1_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_250_Aggr1km_RefSB_Band2");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_250_aggr1km_refsb_band2_radiance,&record->ev_250_aggr1km_refsb_band2_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_500_Aggr1km_RefSB_Band3");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_500_aggr1km_refsb_band3_radiance,&record->ev_500_aggr1km_refsb_band3_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_500_Aggr1km_RefSB_Band4");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_500_aggr1km_refsb_band4_radiance,&record->ev_500_aggr1km_refsb_band4_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_500_Aggr1km_RefSB_Band5");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_500_aggr1km_refsb_band5_radiance,&record->ev_500_aggr1km_refsb_band5_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_500_Aggr1km_RefSB_Band6");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_500_aggr1km_refsb_band6_radiance,&record->ev_500_aggr1km_refsb_band6_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_500_Aggr1km_RefSB_Band7");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_500_aggr1km_refsb_band7_radiance,&record->ev_500_aggr1km_refsb_band7_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band8");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band8_radiance,&record->ev_1km_refsb_band8_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band9");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band9_radiance,&record->ev_1km_refsb_band9_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band10");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band10_radiance,&record->ev_1km_refsb_band10_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band11");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band11_radiance,&record->ev_1km_refsb_band11_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band12");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band12_radiance,&record->ev_1km_refsb_band12_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band13LO");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band13lo_radiance,&record->ev_1km_refsb_band13lo_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band13HI");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band13hi_radiance,&record->ev_1km_refsb_band13hi_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band14LO");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band14lo_radiance,&record->ev_1km_refsb_band14lo_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band14HI");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band14hi_radiance,&record->ev_1km_refsb_band14hi_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band15");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band15_radiance,&record->ev_1km_refsb_band15_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band16");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band16_radiance,&record->ev_1km_refsb_band16_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band17");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band17_radiance,&record->ev_1km_refsb_band17_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band18");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band18_radiance,&record->ev_1km_refsb_band18_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band19");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band19_radiance,&record->ev_1km_refsb_band19_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_RefSB_Band26");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_refsb_band26_radiance,&record->ev_1km_refsb_band26_reflectance);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band20");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band20_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }


    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band21");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band21_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band22");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band22_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band23");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band23_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band24");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band24_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band25");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band25_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band27");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band27_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band28");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band28_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band29");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band29_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band30");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band30_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band31");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band31_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band32");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band32_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band33");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band33_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band34");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band34_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band35");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band35_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD021KM_EV_1KM_Emissive_Band36");
    status = read_band_data(sd_id,sds_name,i_rec,&record->ev_1km_emissive_band36_radiance,NULL);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_MODIS_L1( const caltrack_record_MODIS_L1 const * record ) {
    printf( "Latitude %f\n"          ,  record->lat );
    printf( "Longitude %f\n"         ,  record->lon );
    printf( "Time %e\n"              ,  record->time );
    printf( "Input index [%d,%d]\n"  , record->input_pixel_index[0], record->input_pixel_index[1] );
    printf( "Input File index %d\n"  , record->input_file_index);
    printf( "Input File %s\n"        , record->input_file );
    printf( "Band1\tRadiance=%e\tReflectance=%e\n",    record->ev_250_aggr1km_refsb_band1_radiance, record->ev_250_aggr1km_refsb_band1_reflectance );
    printf( "Band2\tRadiance=%e\tReflectance=%e\n",    record->ev_250_aggr1km_refsb_band2_radiance, record->ev_250_aggr1km_refsb_band2_reflectance );
    printf( "Band3\tRadiance=%e\tReflectance=%e\n",    record->ev_500_aggr1km_refsb_band3_radiance, record->ev_500_aggr1km_refsb_band3_reflectance );
    printf( "Band4\tRadiance=%e\tReflectance=%e\n",    record->ev_500_aggr1km_refsb_band4_radiance, record->ev_500_aggr1km_refsb_band4_reflectance );
    printf( "Band5\tRadiance=%e\tReflectance=%e\n",    record->ev_500_aggr1km_refsb_band5_radiance, record->ev_500_aggr1km_refsb_band5_reflectance );
    printf( "Band6\tRadiance=%e\tReflectance=%e\n",    record->ev_500_aggr1km_refsb_band6_radiance, record->ev_500_aggr1km_refsb_band6_reflectance );
    printf( "Band7\tRadiance=%e\tReflectance=%e\n",    record->ev_500_aggr1km_refsb_band7_radiance, record->ev_500_aggr1km_refsb_band7_reflectance );
    printf( "Band3\tRadiance=%e\tReflectance=%e\n",    record->ev_500_aggr1km_refsb_band3_radiance, record->ev_500_aggr1km_refsb_band3_reflectance );
    printf( "Band8\tRadiance=%e\tReflectance=%e\n",    record->ev_1km_refsb_band8_radiance,         record->ev_1km_refsb_band8_reflectance );
    printf( "Band9\tRadiance=%e\tReflectance=%e\n",    record->ev_1km_refsb_band9_radiance,         record->ev_1km_refsb_band9_reflectance );
    printf( "Band10\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band10_radiance,        record->ev_1km_refsb_band10_reflectance );
    printf( "Band11\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band11_radiance,        record->ev_1km_refsb_band11_reflectance );
    printf( "Band12\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band12_radiance,        record->ev_1km_refsb_band12_reflectance );
    printf( "Band13LO\tRadiance=%e\tReflectance=%e\n", record->ev_1km_refsb_band13lo_radiance,      record->ev_1km_refsb_band13lo_reflectance );
    printf( "Band13HI\tRadiance=%e\tReflectance=%e\n", record->ev_1km_refsb_band13hi_radiance,      record->ev_1km_refsb_band13hi_reflectance );
    printf( "Band14LO\tRadiance=%e\tReflectance=%e\n", record->ev_1km_refsb_band14lo_radiance,      record->ev_1km_refsb_band14lo_reflectance );
    printf( "Band14HI\tRadiance=%e\tReflectance=%e\n", record->ev_1km_refsb_band14hi_radiance,      record->ev_1km_refsb_band14hi_reflectance );
    printf( "Band15\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band15_radiance,        record->ev_1km_refsb_band15_reflectance );
    printf( "Band16\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band16_radiance,        record->ev_1km_refsb_band16_reflectance );
    printf( "Band17\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band17_radiance,        record->ev_1km_refsb_band17_reflectance );
    printf( "Band18\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band18_radiance,        record->ev_1km_refsb_band18_reflectance );
    printf( "Band19\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band19_radiance,        record->ev_1km_refsb_band19_reflectance );
    printf( "Band26\tRadiance=%e\tReflectance=%e\n",   record->ev_1km_refsb_band26_radiance,        record->ev_1km_refsb_band26_reflectance );
    printf( "Band20\tRadiance=%e\n",   record->ev_1km_emissive_band20_radiance );
    printf( "Band21\tRadiance=%e\n",   record->ev_1km_emissive_band21_radiance );
    printf( "Band22\tRadiance=%e\n",   record->ev_1km_emissive_band22_radiance );
    printf( "Band23\tRadiance=%e\n",   record->ev_1km_emissive_band23_radiance );
    printf( "Band24\tRadiance=%e\n",   record->ev_1km_emissive_band24_radiance );
    printf( "Band25\tRadiance=%e\n",   record->ev_1km_emissive_band25_radiance );
    printf( "Band27\tRadiance=%e\n",   record->ev_1km_emissive_band27_radiance );
    printf( "Band28\tRadiance=%e\n",   record->ev_1km_emissive_band28_radiance );
    printf( "Band29\tRadiance=%e\n",   record->ev_1km_emissive_band29_radiance );
    printf( "Band30\tRadiance=%e\n",   record->ev_1km_emissive_band30_radiance );
    printf( "Band31\tRadiance=%e\n",   record->ev_1km_emissive_band31_radiance );
    printf( "Band32\tRadiance=%e\n",   record->ev_1km_emissive_band32_radiance );
    printf( "Band33\tRadiance=%e\n",   record->ev_1km_emissive_band33_radiance );
    printf( "Band34\tRadiance=%e\n",   record->ev_1km_emissive_band34_radiance );
    printf( "Band35\tRadiance=%e\n",   record->ev_1km_emissive_band35_radiance );
    printf( "Band36\tRadiance=%e\n",   record->ev_1km_emissive_band36_radiance );
}
