/*
*  C Implementation: caltrack_reader_parasol_rb2
*
* Description:
*   Set the structure and the functions to read a record of a PARASOL RB2 CALTRACK_UNIT product
*
* Author:
*   CGTD-ICARE/UDEV Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* License :
*   This file must be used under the terms of the CeCILL.
*   This source file is licensed as described in the file COPYING, which
*   you should have received as part of this distribution.  The terms
*   are also available at
*   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#include "caltrack_reader_parasol_rb2.h"

int read_caltrack_record_PARASOL_RB2( const char* filename, const int i_rec, caltrack_record_PARASOL_RB2 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,NB_PARASOL_DIRECTION};
    /* name of an SDS */
    char sds_name[MAX_NC_NAME];
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        return FAIL;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* grid index */
    strcpy(sds_name,"P3L2TRGB_Input_Index");
    edges_2D[1]=2;
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->input_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Cover");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_cover, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Directional_Cloud_Cover");
    edges_2D[1]=NB_PARASOL_DIRECTION;
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->directional_cloud_cover, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Rayleigh_Pressure");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_rayleigh_pressure, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Rayleigh_Pressure_Angular_Stddev");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_rayleigh_pressure_angular_stddev, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Oxygen_Pressure");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_oxygen_pressure, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Oxygen_Pressure_Angular_Stddev");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_oxygen_pressure_angular_stddev, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Optical_Thickness");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_optical_thickness, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Optical_Thickness_Stddev");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_optical_thickness_stddev, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cloud_Phase");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->cloud_phase, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Water_Vapor_Column");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->water_vapor_column, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Mean_Albedo");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->mean_albedo, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Albedo_Spatial_Stddev");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->albedo_spatial_stddev, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Albedo_Angular_Stddev");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->albedo_angular_stddev, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Surface_Type_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->surface_type_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Cos_Solar_Zenith_Angle");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cos_solar_zenith_angle, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Scene_Albedo_Angular_Stddev");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->scene_albedo_angular_stddev, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TRGB_Optical_Thickness_Homogeneity_Coef");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->optical_thickness_homogeneity_coef, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
};

void print_caltrack_record_PARASOL_RB2 ( const caltrack_record_PARASOL_RB2 const * record ) {
    printf( "Latitude %f\n",                                    record->lat );
    printf( "Longitude %f\n",                                   record->lon );
    printf( "Time %e\n",                                        record->time );
    printf( "Grid index [%d,%d]\n",                             record->input_index[0], record->input_index[1] );
    printf( "Cloud Cover %e\n",                                 record->cloud_cover );
    printf( "Directional Cloud Cover [%e %e ... %e %e]\n",      record->directional_cloud_cover[0],
                                                                record->directional_cloud_cover[1],
                                                                record->directional_cloud_cover[NB_PARASOL_DIRECTION-2],
                                                                record->directional_cloud_cover[NB_PARASOL_DIRECTION-1]);
    printf( "Cloud Rayleigh Pressure %e\n",                     record->cloud_rayleigh_pressure );
    printf( "Cloud Rayleigh Pressure Stddev %e\n",              record->cloud_rayleigh_pressure_angular_stddev );
    printf( "Cloud Oxygen Pressure %e\n",                       record->cloud_oxygen_pressure );
    printf( "Cloud Oxygen Pressure Stddev %e\n",                record->cloud_oxygen_pressure_angular_stddev );
    printf( "Cloud Optical Thickness %e\n",                     record->cloud_optical_thickness );
    printf( "Cloud Optical Thickness Stddev %e\n",              record->cloud_optical_thickness_stddev );
    printf( "Cloud Phase %u\n",                                 record->cloud_phase );
    printf( "Water Vapor Column %e\n",                          record->water_vapor_column );
    printf( "Mean Albedo %e\n",                                 record->mean_albedo );
    printf( "Albedo Spatial Standard Deviation %e\n",           record->albedo_spatial_stddev );
    printf( "Albedo Angular Standard Deviation %e\n",           record->albedo_angular_stddev );
    printf( "Surface Type Index %d\n",                          record->surface_type_index );
    printf( "Cosine of Solar Zenith Angle %e\n",                record->cos_solar_zenith_angle );
    printf( "Scene Albedo Angular Standard Deviation %e\n",     record->scene_albedo_angular_stddev );
    printf( "Optical Thickness Homogeneity Coefficient %e\n",   record->optical_thickness_homogeneity_coef );

}
